/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gems;

import java.io.*;
import java.util.*;
import javax.swing.tree.*;
import javax.swing.JPopupMenu;
import java.awt.event.*;
import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.DebugStream;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.collection.BasicCollectionConfiguration;
import org.w3c.dom.*;
/**
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 * Modified 2/02/05 by Matthew Whyte
 */
public class GEMSNode
    extends DefaultMutableTreeNode{
    private AttributeTableModel model = null;
    public int type = 0;
    private String text = null;
    static final public int COLLECTION = 0;
    static final public int ELEMENT    = 1;
    static final public int PROFILER   = 2;
    static final public int ROOT       = 3; 
    static final public int SET        = 4;
    //static final public int SUBELEMENT = 5; 

    public GEMSNode() {
	this.type = ROOT;
    }

    public GEMSNode(int type, Object object, GEMSNode parent) {
	this.userObject = object;
	this.parent = parent;
	this.type = type;
	this.text = null;
    }

    public Enumeration children() {
	if(children == null) {
	    mapChildren();
	}		  
	return children.elements();
    }

    public boolean getAllowsChildren() {
	return true;
    }

    public TreeNode getChildAt(int index) {
	if(children == null) {
	    mapChildren();
	}
	return (TreeNode) children.get(index);
    }
    public int getChildCount() {
	if(children == null) {
	    mapChildren();
	}
	return children.size();
    }
    public ElementWrapper getElement() {
	ElementWrapper result = null;
	if(type == ELEMENT) {
	    result = (ElementWrapper) userObject;
	}
	return result;	
    }

    public int getIndex(TreeNode node) {
	if(children == null) {
	    mapChildren();
	}
	return children.indexOf(node);
    }
    public AttributeTableModel getModel() {
	return model;
    }
    public TreeNode getParent() {
	return parent;
    }
    public MetadataSet getSet() {
	MetadataSet result = null;
	if(type == SET) {
	    result = (MetadataSet) userObject;
	}
	return result;
    }
    public int getType() {
	return type;
    }
    public boolean isLeaf() {
	if(children == null) {
	    mapChildren();
	}
	return children.size() == 0;
    }
    public void setModel(AttributeTableModel model) {
	this.model = model;
    }

    public String toString()
    {
	if(text == null)
	    {
		if(userObject != null)
		    {
			if(userObject instanceof ElementWrapper)
			    {
				text = ((ElementWrapper)userObject).getName();
			    }
			else
			    {
				text = userObject.toString();
			    }
		    }
		else
		    {
			text = "error";
		    }
	    }
	return text;
    }

    public void mapChildren()
    {
	children = new Vector();

	// How we build children depends on the node type
	if (type == SET)
	    {
		// Add the elements as children
		MetadataSet set = (MetadataSet) userObject;
		ArrayList elements = set.getElements();
		for (int i = 0; i < elements.size(); i++)
		    {
			children.add(new GEMSNode(ELEMENT, new ElementWrapper((Element) elements.get(i)), this));
		    }
	    }
	if (type == ELEMENT) //Add subelements. As elements and subelements are identical, this process continues.
	    {
		ArrayList subelements = this.getElement().getSubelements();
		for(int j = 0; j < subelements.size(); j++)
		    {
			children.add(new GEMSNode(ELEMENT, new ElementWrapper((Element)subelements.get(j)), this));
		    }
	    }
    }
    
   class GemsNodeMouseAdapter
    extends MouseAdapter {
       
      public void mouseClicked(MouseEvent e) {
        JPopupMenu setPopup = new JPopupMenu("asdf");
         if (e.isPopupTrigger()) {       
	     setPopup.show(e.getComponent(),
                           e.getX(), e.getY());
	 }
    }    
    
    public void mouseEntered(MouseEvent e) {
    }    
  
    public void mouseExited(MouseEvent e) {
    }
    
    public void mousePressed(MouseEvent e) {
         JPopupMenu setPopup = new JPopupMenu("asdf");
         if (e.isPopupTrigger()) {               
	     setPopup.show(e.getComponent(),
                           e.getX(), e.getY());
	 }
    }
       
    public void mouseReleased(MouseEvent e) {
         JPopupMenu setPopup = new JPopupMenu("asdf");
         if (e.isPopupTrigger()) {                  
	     setPopup.show(e.getComponent(),
                           e.getX(), e.getY());     
	 }
    }  
   }
}
